{*************************************************************}
{*                Unit FaMath  v1.0                          *}
{*    Copyright (c) 2000 by Dmitry Beloshistov [-=BDS=-]     *}
{*************************************************************}
Unit FaMath;

Interface

Function Tan(X:Real):Real;
Function Cot(X:Real): Real;
Function Sec(X:Real): Real;
Function Cosec(X:Real): Real;
Function Sinh(X:Real): Real;
Function Cosh(X:Real): Real;
Function Tanh(X:Real): Real;
Function Coth(X:Real): Real;
Function Sech(X:Real): Real;
Function Cosech(X:Real): Real;
Function Arcsin(X:Real):Real;
Function Arccos(X:Real):Real;
Function Arccot(X:Real): Real;
Function Arcsec(X:Real): Real;
Function Arccosec(X:Real): Real;
Function Arcsinh(X:Real): Real;
Function Arccosh(X:Real): Real;
Function Arctanh(X:Real): Real;
Function Arccoth(X:Real): Real;
Function Log(Base,Innumber:Real):Real;
Function Log10( X: Real ) : Real;
Function Root(Innumber,Theroot:Real):Real;
Function Percent(Current,Max:Longint):Longint;
Function Power( X: Real; Y: Real ) : Real;
Function Powerint( X: Integer; Y: Integer ) : Integer;
Function Poweri( X: Real; I: Integer ) : Real;
Function Pow10( Power : Integer ) : Real;
Function Setbit(L:Longint; Bitnum:Word):Longint;
Function Clrbit(L:Longint; Bitnum:Word):Longint;
Function Testbit(L:Longint; Bitnum:Word):Boolean;
Function Factorial(L:Longint):Longint;

implementation

function Log(Base,InNumber:real):real;
{ log }
begin
  Log:=ln(InNumber)/ln( Base );
end;

Function Log10( X: Real ) : Real;
{ lg  }
Begin
 If X <= 0.0 Then Log10:=0.0 Else Log10 := Ln( X ) * (1/Ln(10));
End;

Function Power( X: Real; Y: Real ) : Real;
{ X^Y }
Begin
   If X > 0 Then Power := Exp( Y * Ln( X ) ) Else Power := 0.0;
End;

Function Root(Innumber,Theroot:Real):Real;
{ Root }
Begin
  Root := Power( Innumber, ( 1.0 / Theroot ) );
End;

Function Percent(Current,Max:Longint):Longint;
{ Repcent }
Begin
  Percent:=Trunc(100*(Current/Max));
End;

Function Powerint( X: Integer; Y: Integer ) : Integer;
{ X^Y for integer }
Var N,I : Integer;
Begin
 N:=X;
 For I:=1 To Y-1 Do X:=X * N;
 Powerint:=X;
End;

Function Poweri( X: Real; I: Integer ) : Real;
{ X^I for real }
Var Temp: Real;
    Absi: Integer;
Begin
 If I < 0 Then Begin
                 I := -I;
                 If X <> 0.0 Then X := 1.0 / X;
               End;
   Temp := 1.0;
   While( I > 0 ) Do
      Begin
         While (Not Odd(I)) Do
            Begin
               I := I Div 2;
               X := X * X;
            End;
         I:=I-1;
         Temp := Temp * X;
      End;
   Poweri := Temp;
End;

Function Pow10( Power : Integer ) : Real;
{ X^10 }
Var
   Temp   : Real;
   I      : Integer;
   Abspow : Integer;
   X      : Real;
Begin
   X := 10.0;
   If Power < 0 Then  Begin Power := -Power;  X := 0.1; End;
   Temp := 1.0;
   While( Power > 0 ) Do
      Begin
         While ( Not Odd( Power ) ) Do
            Begin
               Power := Power Div 2;
               X     := X * X;
            End;
          Power := Power - 1;
         Temp  := Temp  * X;
      End;
   Pow10 := Temp;
End;

Function Tan(X: Real): Real;
{ argument X is in radians }
Begin
   Tan := Sin(X)/Cos(X);
End;

Function Cot(X:Real): Real;
{ cotangent, x is in radians }
Begin
   Cot := 1/Tan(X);
End;

Function Sec(X:Real): Real;
{ secant, x is in radians }
Begin
   Sec := 1/Cos(X);
End;

Function Cosec(X:Real): Real;
{ cosecant, x is in radians }
Begin
   Cosec := 1/Sin(X);
End;

Function Sinh(X:Real):Real;
{ hyperbolic sin }
Begin
   Sinh := (Exp(X)-Exp(-X))/2;
End;

Function Cosh(X:Real): Real;
{ hyperbolic cos }
Begin
   Cosh := (Exp(X)+Exp(-X))/2;
End;

Function Tanh(X:Real): Real;
{ hyperbolic tan }
Begin
   Tanh := Sinh(X)/Cosh(X);
End;

Function Coth(X: Real): Real;
{ hyperbolic cotangent }
Begin
   Coth :=Sinh(X)/Cosh(X);
End;

Function Sech(X:Real): Real;
{ hyperbolic secant }
Begin
   Sech := 1/Cosh(X);
End;

Function Cosech(X:Real): Real;
{ hyperbolic cosecant }
Begin
   Cosech := 1/Sinh(X);
End;

Function Arcsin(X:Real):Real;
{ inverse of sin, return value is in radians }
Begin
   If Abs(X)=1.0  Then Arcsin := X*Pi/2
   Else  Arcsin := Arctan(X/Sqrt(-Sqr(X)+1));
End;

Function Arccos(X:Real):Real;
{ inverse of cos, return value is in radians }
Begin
   If X = 1.0 Then Arccos := 0
   Else If X = -1.0 Then Arccos :=Pi
        Else Arccos := -Arctan(X/Sqrt(-Sqr(X)+1))+Pi/2;
End;

Function Arccot(X:Real): Real;
{ inverse of cot, return value is in radians }
Begin
   Arccot := Arctan(1/X);
End;

Function Arcsec(X:Real): Real;
{ inverse of secant, return value is in radians }
Begin
   Arcsec := Arccos(1/X);
End;

Function Arccosec(X:Real): Real;
{ inverse of cosecant, return value is in radians }
Begin
   Arccosec := Arcsin(1/X);
End;

Function Arcsinh(X:Real): Real;
{ inverse of hyperbolic sin }
Begin
   Arcsinh := Ln(X + Sqrt(X*X+1));
End;

Function Arccosh(X:Real): Real;
{ inverse of hyperbolic cos}
Begin
   Arccosh := Ln(X + Sqrt(X*X-1));
End;

Function Arctanh(X:Real): Real;
{ inverse of hyperbolic tan }
Begin
   Arctanh := Ln((1+X)/(1-X))/2;
End;

Function Arccoth(X:Real): Real;
{ inverse of hyperbolic cotangent }
Begin
   Arccoth := Ln((X+1)/(X-1))/2;
End;

Function SetBit(L:longint; BitNum:word):longint;
{ set bit to 1 }
begin
 SetBit:=L or (1 shl BitNum);
end;

Function ClrBit(L:Longint; BitNum:word):longint;
{ set bit to 0 }
begin
 ClrBit:=L and not(1 shl BitNum);
end;

Function TestBit(L:Longint; BitNum:word):boolean;
{ true if bit is 1 else false }
begin
 TestBit:=((L and (1 shl BitNum))<>0);
end;

Function Factorial(L:Longint):Longint;
{ factorial }
var i:longint;
    Result:Longint;
begin
 Result:=1;
 for i:=2 to L do Result:=Longint(Result*I);
 Factorial:=Result;
end;

End.
