{*********************************************************}
{*                   Unit IniFile v1.0                   *}
{*      (c) 1998 by Dmitry Beloshistov [-=BDS=-]         *}
{*             Windows INI files processing              *}
{*********************************************************}

Unit IniFile;
Interface
Uses Objects;

type
    PIniFile=^TIniFile;
    TIniFile=object(TObject)
              IniFile:PString;
              F:Text;
              OldFAttr:Word;
              Error:Byte;
              Constructor Init(AIniFile:String);
              Destructor Done; virtual;
              Function ReadStrValue(Section,Param,DefValue:String):String;
              Function ReadIntValue(Section,Param:String; DefValue:Longint):longint;
              Function ReadFloatValue(Section,Param:String; DefValue:Real):Real;
              Function SectionExists(Section:string):boolean;
              Function ParamExists(Section,Param:String; var IniStr:String):boolean;
              Procedure WriteStrValue(Section,Param,Value:String);
              Procedure WriteIntValue(Section,Param:String;Value:longint);
              Procedure WriteFloatValue(Section,Param:String;Value:real);
             end;

Implementation
Uses Dos,FaFuncs;

Constructor TIniFile.Init(AIniFile:String);
begin
 Inherited Init;
 Error:=0;
 if Trim(AIniFile)='' then Error:=1
 else begin
       IniFile:=NewStr(AIniFile);
       Assign(F,IniFile^);
       GetFAttr(F,OldFAttr); SetFAttr(F,Archive);
       {$I-} reset(F); {$I+}
       if IOResult<>0 then Error:=2;
      end;
end;

Destructor TIniFile.Done;
begin
 if Error<>1 then DisposeStr(IniFile);
 if Error<>2 then begin Close(F); SetFAttr(F,OldFAttr); end;
 Inherited Done;
end;

Function IsSection(S:String):boolean;
begin
 if S<>'' then IsSection:=(S[1]='[')
 else IsSection:=False;
end;

Function GetParamValue(S:String):String;
begin
 while (S<>'') and (S[1]<>'=') do Delete(S,1,1);
 Delete(S,1,1);
 GetParamValue:=S;
end;

Function TIniFile.SectionExists(Section:String):boolean;
var S:String;
    Flag:boolean;
begin
 Section:='['+UpperCase(Trim(Section))+']';
 Flag:=False;
 {$I-} Reset(F); {$I+}
 if IOResult<>0 then Error:=2
 else while not eof(F) do
        begin
         readln(f,S);
         S:=UpperCase(DelComment(Trim(S)));
         if Pos(Section,S)>0 then begin Flag:=True; break; end;
       end;
 SectionExists:=Flag;
end;

Function TIniFile.ParamExists(Section,Param:String; var IniStr:String):boolean;
var S:String;
begin
 IniStr:=''; ParamExists:=False;
 if SectionExists(Section) then
  begin
   Param:=UpperCase(Param)+'=';
   while not Eof(F) do
     begin
      readln(F,S); S:=Trim(S);
      if IsSection(S) then break;
      S:=DelComment(S); IniStr:=S;
      S:=DelSpace(UpperCase(S));
      if Pos(Param,S)>0 then begin ParamExists:=True; break; end;
    end;
  end;
end;

Function TIniFile.ReadStrValue(Section,Param,DefValue:String):String;
var S:String;
begin
 if ParamExists(Section,Param,S) then ReadStrValue:=GetParamValue(S)
 else ReadStrValue:=DefValue;
end;

Function TIniFile.ReadIntValue(Section,Param:String; DefValue:Longint):longint;
var S:String;
    N:longint;
begin
 if ParamExists(Section,Param,S) then
  begin
   N:=StrToInt(GetParamValue(S));
   if N=MaxLongint then ReadIntValue:=DefValue;
  end
 else ReadIntValue:=DefValue;
end;

Function TIniFile.ReadFloatValue(Section,Param:String; DefValue:real):real;
var S:String;
    N:real;
begin
 if ParamExists(Section,Param,S) then
  begin
   N:=StrToFloat(GetParamValue(S));
   if N=MaxLongint then ReadFloatValue:=DefValue;
  end
 else ReadFloatValue:=DefValue;
end;

Procedure TIniFile.WriteStrValue(Section,Param,Value:String);
var FBuf:Text;

    Procedure AppendSectionParam;
    begin
      Close(F);
      {$I-} Append(F); {$I+}
      if IOResult<>0 then begin exit; Error:=2; end;
      writeln(F,' ');
      writeln(F,'[',Section,']');
      writeln(F,Param,'=',Value);
      Close(F); Reset(F);
      Error:=0;
    end;

    Procedure WriteSection;
    var S,SBuf:String;
    begin
     Assign(FBuf,'#$tmp$#.ini');
     {$I-} rewrite(FBuf); {$I+}
     if IOResult<>0 then begin Error:=2; exit; end;
     {$I-} Reset(F); {$I+}
     if IOResult<>0 then begin Error:=2; exit; end;
     Section:='['+UpperCase(Trim(Section))+']';
     while not eof(F) do
      begin
       readln(f,S); Sbuf:=S;
       S:=UpperCase(DelComment(Trim(S)));
       writeln(FBuf,SBuf);
       if Pos(Section,S)>0 then break;
      end;
    end;

   procedure WriteParam;
   var S,Sbuf:String;
   begin
    Param:=Param+'=';
    while not Eof(F) do
      begin
       readln(F,S); SBuf:=S; S:=Trim(S);
       if IsSection(S) then begin
                              writeln(FBuf,Param,Value);
                              writeln(FBuf,S); break;
                            end;
       S:=DelComment(S); S:=DelSpace(UpperCase(S));
       if Pos(UpperCase(Param),S)>0 then begin writeln(FBuf,Param,Value); break; end
       else Writeln(FBuf,SBuf);
      end;
     while not eof(F) do begin Readln(F,S); writeln(FBuf,S); end;
       close(F); Close(FBuf);
       Erase(F); Rename(FBuf,IniFile^);
       reset(F);
     end;

begin
 if Error=1 then exit; {no file!!!}
 if Error=2 then
  begin
   {$I-} rewrite(F); {$I+}
   if IOResult<>0 then begin Error:=2; exit; end
   else AppendSectionParam;
  end
 else begin             {Error=0}
       if not SectionExists(Section) then AppendSectionParam
       else begin WriteSection; WriteParam; end;
      end;
end;

Procedure TIniFile.WriteIntValue(Section,Param:String;Value:longint);
begin
 WriteStrValue(Section,Param,IntToStr(Value));
end;

Procedure TIniFile.WriteFloatValue(Section,Param:String;Value:real);
begin
 WriteStrValue(Section,Param,FloatToStr(Value));
end;
{**********************************************************************}
{*                               DEMO                                 *}
{**********************************************************************}
(*----------------------------------------------------------------------
var Inf:TiniFile;
begin
 Inf.Init('win.ini');
 writeln(Inf.ReadStrValue('windows','device','Value not found!!!'));
 Inf.WriteStrValue('windows','device','NEWVALUE FOR DEVICE!!!');
 Inf.WriteStrValue('windows','IsItTestParam','Value for test Param!!!');
 Inf.WriteStrValue('ItIsNewSection','NewSectionParam','NewSectionParamValue');
 Inf.Done;
End.
----------------------------------------------------------------------*)
end.